"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (_ref) {
  var t = _ref.types;

  return {
    pre: function pre() {
      this.cache = new Map();
    },

    visitor: {
      StringLiteral: function StringLiteral(path, state) {
        // don't replace import/export strings
        if (path.parentPath.isModuleDeclaration()) {
          return;
        }

        var minimumStringLength = state.opts.minimumStringLength;
        var value = path.node.value;

        // minimumStringLength option to run the transform
        if (value.length < (minimumStringLength || 0)) {
          return;
        }

        // TODO: should the key just be the string value?
        if (!this.cache.has(value)) {
          this.cache.set(value, { indexes: 1, firstPath: path });
        } else {
          var cachedValue = this.cache.get(value);
          cachedValue.indexes++;

          var uid = void 0;
          // after the first duplicate value
          // create the new variable declaration
          if (cachedValue.indexes === 2) {

            // TODO: can use something else
            uid = path.scope.generateUidIdentifier("a");

            if (!cachedValue.ref) {
              cachedValue.ref = uid;
            }
            cachedValue.firstPath.replaceWith(uid);

            var newString = t.stringLiteral(value);
            // create private property so that string isn't replaced
            // with a reference to itself
            newString._created = true;

            var declar = t.variableDeclaration("var", [t.variableDeclarator(uid, newString)]);
            var childOfProgramPath = path.findParent(function (path) {
              return path.parentPath.isProgram();
            });
            childOfProgramPath.insertBefore(declar);
          } else {
            uid = cachedValue.ref;
          }

          // change the string reference with the
          // newly created variable reference
          if (!path.node._created) {
            path.replaceWith(uid);
          }
        }
      } // StringLiteral

    }
  };
};